// the gameworld class handling everything thats in the gameworld of the game

class GameWorld 
{
  // Variables for the mircrophone lines that the players chose
  TargetDataLine lineP1, lineP2;

  // An array for each line to store all the read bytes in
  int numBytesReadLine1, numBytesReadLine2;
  byte[] data1, data2;
  int dataSize = 1000;
  byte lastByte = 0;
  boolean deathToggle = false;

  // Variables to store the levels of each microphone in
  public float accum1, accum2;

  //global-forces:
  PVector gravity = new PVector(0.0, 9.81), 
    friction = new PVector(0, 0), 
    wind = new PVector(0, 0);

  // spider & silk:
  Spider spidey;

  // data to implement the scrolling of the background
  float millisNow;
  int millisDrawWasLastCalled;

  // sprites variables
  ArrayList<Sprite> objects = new ArrayList<Sprite>(); // all items (enemies and plus points) are stored in here
  float xStart, yStart; //used to store the starting position of each sprite when spawning them later

  //Counter
  Counter blue, pink;
  int cb = 0; // stores the points the blue player has collected 
  int cp = 0; // stores the points the pink player has collected 

  // Image Scrolling Setup
  PImage img, map;
  int timeElapsed;
  int delay;

  // Constructor of the GameWorld in which the lines of the microphones get received and stored.
  GameWorld(TargetDataLine _lineP1, TargetDataLine _lineP2) 
  {
    lineP1 = _lineP1;
    lineP2 = _lineP2;

    // The microphone lines get set up with the micSetup() method
    micSetup();  

    // Byte arrays with fixed sizes get opened for the line values to store in
    data1 = new byte[dataSize];
    data2 = new byte[dataSize];

    // Then the lines get openend and started to read fron
    lineP1.start(); 
    lineP2.start();

    // Initializing the spider object
    spidey = new Spider(this);

    millisDrawWasLastCalled = millis();

    // Loading Images and Map Loading
    img = loadImage("Level_01_04.png");
    map = loadImage("Map_Level01_04.png");
    map.loadPixels();
    blue = new Counter ( this, false); //false and true only indicate if the counter object is pink or not 
    pink = new Counter (this, true);
    spawnSprites();                    // checks the map array for certain colors and spawns sprites at the right locations
  }

  void draw() 
  {  
    // MAP DRAWING
    int millisNow = millis();
    float timeElapsed = (millisNow - millisDrawWasLastCalled) / 1000.0;

    millisNow = millis();
    int y = int((timeElapsed*100)*.9); //framecount: total frames after running program

    delay = int(height/2.5); //this makes sure the screen doesn't immediately starts scrolling so the player can orient himself

    if (y < delay) {             // don't start scrolling until y exceeds the delay
      translate (0, 0);
    } else if (y - delay < img.height-height) { 
      translate(0, -(y - delay));                          // translate the coordinate system by minus y, since y is connected to the timeElapsed
    } else {                                               // there is steady scrolling down
      translate(0, - (img.height - height));               // makes sure the image doesn't scroll off-screen
    }

    spidey.move();

    image(img, 0, 0);

    // draw the Counter objects at fixed positions on the screen 
    // since the coordinate system is translated in 3 different ways there a 3 different cases where to draw the counter objects

    if (y < delay) {                                // draws counter at bottom of screen at the start
      translate (0, 0);
      pink.display(height - 80);
      blue.display(height - 80);
    } else if (y - delay < img.height-height) {     // draws counter at bottom of the screen during scrolling

      pink.display((y - delay) + (height - 80));    
      blue.display((y - delay) + (height - 80));
    } else {

      pink.display(img.height -80);                 // draws the counter at the bottom of the frame after the scrolling has stopped
      blue.display(img.height  - 80);
    }

    if (objects != null) {                          // draws all objects in the Sprite Array, they were initialised in spwanSprites()
      for (Sprite object : objects) {
        object.display();
        object.animate();
      }
    }

    //checking all collisions between spidey and the other game components
    coll_object();          // spidey & objects in Sprite Array
    coll_world(spidey);     // spidey & the gameworld (the pixel array saved in map)
    coll_frame(y);          // spidey & the edges of the window

    //SPIDEY DRAWING
    spidey.display();
    spidey.animate();

    // INPUT UPDATE
    updateLevels();
  }

  // SPAWNING THE SPRITES ACCORDING TO THE BLUE DOTS IN THE MAP

  void spawnSprites() {


    // i = y * map.width + x; 
    // x = index % Width;
    //y = (index - x) / Width;

    // check every Pixel of the map array once

    for (int i = 0; i < map.pixels.length; i++) {

      // check if the dot is 100 % blue (green value 0 and blue 255
      // it's important that green is 0 as there are cyan dots also
      //                         g                               b
      if  (((map.pixels[i]  >> 8) & 0xff) == 0 && ((map.pixels[i]  >> 0) & 0xff) == 255) {

        xStart = (i % map.width);                // set the x Position of the sprite at the first blue pixel found
        yStart = ((i -xStart) / map.width);      // set the y Position of the sprite at the first blue pixel found
        objects.add(new Sprite(loadFrames("bat", 8, 100, 100), xStart, yStart, random(0.3, 1), getDirection(xStart), 0));

        if (i != (map.pixels.length - 20)) {     // make sure there 
          i = i + 20 * map.width + 20;           //jump ahaead to avoid checking the same dot twice
        }
      }

      // cyan for blue flowers ( Player 1) 
      //                         g                               b
      if  (((map.pixels[i]  >> 8) & 0xff) == 255 && ((map.pixels[i]  >> 0) & 0xff) == 255) {

        xStart = (i % map.width) - 50; 
        yStart = ((i -xStart) / map.width) - 50;
        objects.add(new Sprite(loadFrames("blue_flower", 8, 35, 57), xStart, yStart, 0.01, false, 1));    
        if (i != (map.pixels.length - 20)) {
          i = i + 20 * map.width + 20; //jump ahaead to avoid checking the same dot twice
        }
      }
      // yellow for pink flowers 
      if  (((map.pixels[i]  >> 16) & 0xff) == 255 && ((map.pixels[i]  >> 8) & 0xff) == 255) {

        xStart = (i % map.width) - 50; 
        yStart = ((i -xStart) / map.width) - 50;
        objects.add(new Sprite(loadFrames("pink_flower", 8, 35, 57), xStart, yStart, 0.01, false, 2));    
        if (i != (map.pixels.length - 20)) {
          i = i + 20 * map.width + 20; //jump ahaead to avoid checking the same dot twice
        }
      }
    }
  }

  boolean getDirection(float x) {      // specifically for the bats flyng across the screen
    if (x > width/2) {                 // if the blue dot is to the right of the screen the bat faces and flies to the left
      return true;
    } else {
      return false;
    }
  }

  int score() {                         // returns an int according to which player has the higher score
    if (cb > cp) {
      return 1;
    }
    if (cp > cb) {
      return 2;
    } else return 0;
  }


  void coll_frame(float y) {


    if (y > img.height - height) {               
      y = -(img.height - height);
    }

    // spider gets initialised at 0/0 

    if (spidey.yPos != 0 && spidey.yPos < (y - delay)) {  // checks if spidey collides with the top of the screen 
      //println("Collided with Top");
      gameMode = new GameEndsMode("You both lose!", #2b1e4b, lineP1, lineP2, score());
    }

    if (spidey.xPos  < 0)                                 // check collision with left edge
    {
      // println("Collided with left side");
      gameMode = new GameEndsMode("You both lose!", #2b1e4b, lineP1, lineP2, score());
    }

    if ((spidey.xPos + spidey.sprite_width) == width)     // check collision with right edge
    {
      //println("Collided with right side");
      gameMode = new GameEndsMode("You both lose!", #2b1e4b, lineP1, lineP2, score());
    }
  }

  void coll_object() {                                    //checks collisions with all elements of the Sprite Array
    //consequences of collision according to object type
    if (objects != null) {
      for (Sprite object : objects) {

        if (spidey.spriteCollision(spidey, object)) {

          if (object.type == 0) {                        // type 0 is enemy

            //println("Collided with Enemy");
            gameMode = new GameEndsMode("You both lose!", #2b1e4b, lineP1, lineP2, score());
          }

          if (object.type == 1) {                        // type 1 are the blue flowers, collected by Player 1
            cb = cb +1;                                  // the longer the player hovers over the flower the more the counter goes up
            blue.saveNum(cb);                            // saves the number in the Counter Class so it can be displayed on the cloud
          }

          if (object.type == 2) {                        // type 2 are the pink flowers, collected by Player 2
            cp = cp+1;
            pink.saveNum(cp);
          }
        }
      }
    }
  }

  void coll_world(Sprite r) {

    spidey.silkCollision();

    // search the array in the area the sprite rectangle is over
    // top left corner sprite.xPos, top right corner: sprite.xPos + sprite_width

    int rx = int(r.xPos);
    int rw = rx + int(r.sprite_width);
    int ry = int(r.yPos);
    int rh = ry + int(r.sprite_height);

    for (int x = rx; x < rw; x++) {                          // go over every Pixel the Sprite is currently covering in x
      for (int y = ry; y < rh; y++) {                        // go over every Pixel the Sprite is currently covering in y
        int currentPixel = map.pixels[ y * map.width + x];

        // check the colors the Spidey rectangle is currently over

        //              r                                           g                                         b            
        //collsion with world if the pixel is red

        if  (((currentPixel >> 16) & 0xff) == 255 && ((currentPixel  >> 8) & 0xff) == 0 && ((currentPixel  >> 0) & 0xff) == 0) {  // RED
          delay(60); 
          gameMode = new GameEndsMode("You both lose!", #2b1e4b, lineP1, lineP2, score());
        }

        //collsion with the aim if the pixel is green

        if  (((currentPixel >> 16) & 0xff) == 0 && ((currentPixel  >> 8) & 0xff) == 255 && ((currentPixel  >> 0) & 0xff) == 0) {  // GREEN
          gameMode = new GameEndsMode("You win!", #f09030, lineP1, lineP2, score());
        }
      }
    }
  }

  // Method to set up the mics and get them ready to read from
  void micSetup()
  {
    // Setting the audio format that we want the lines to be read with
    AudioFormat format = new AudioFormat(11025, 8, 1, true, true);

    // Obtain and open the line.
    try 
    {
      // println("Opening line 1: " + lineP1); debug
      lineP1.open(format);
      // println("Opening line 2: " + lineP2); debug
      lineP2.open(format);
    }
    // Error Handling
    catch (LineUnavailableException ex) 
    {
      println(ex.toString());
      exit();
    }
    catch (IllegalArgumentException ex)
    {
      println(ex.toString());
      exit();
    }
    print("Line Setup Complete");
  }

  // Method to update the levels, read and set the wind forces according to them
  void updateLevels()
  {
    // checking if there is enough bytes to read from
    while (lineP1.available() >= data1.length)
    {
      // getting the current byte value of the line
      numBytesReadLine1 = lineP1.read(data1, 0, data1.length);
    }

    // same as above for the second line
    while (lineP2.available() >= data2.length)
    {
      numBytesReadLine2 = lineP2.read(data2, 0, data2.length);
    }

    accum1 = 0;
    accum2 = 0;

    // Going through the byte arrays, reading the bytes and wheighting them to get the total level
    // Division by 128 to get values from 0. to 1.
    for (int i = 0; i < dataSize; i++)
    {
      accum1 += (data1[i] / 128.0) * (data1[i] / 128.0);
      accum2 += (data2[i] / 128.0) * (data2[i] / 128.0);
    }

    // Input scaling to get desired level values to be used
    accum1 = (sqrt(accum1) / dataSize) * 128.0 * 15;
    accum2 = (sqrt(accum2) / dataSize) * 128.0 * 15;

    // Setting the wind forces and behaviour
    wind.x -= (accum1/3); //substraction with x value of wind for right wind
    wind.x += (accum2/3); //addition with x value of wind for left wind
  }

  // handling keyPresses, debbuging purposes only
  void keyPressed()
  {
    spidey.keyPressed();
  }
  void keyReleased()
  {
    spidey.keyReleased();
  }
}
