// A class that handles the menu shown when starting the game
class MenuMode extends GameMode 
{
  Sprite spider;
  
  // An array to store all the found lines in
  ArrayList<LineGetter> lineGetters = new ArrayList<LineGetter> ();  
  TargetDataLine lineP1, lineP2;    
  
  // Variables to store the line informations for each player in
  LineGetter player1Line;
  LineGetter player2Line;
  
  // Variables to store the index of the lines in the linegetter array for each player
  int player1LineIndex;
  int player2LineIndex; 

  // Buttons needed
  Button button0, buttonP1, buttonP2;

  MenuMode()
  {
    spider = new Sprite(loadFrames("Spidey", 8, 100, 50), width/2 - 50, height/2 -150, 0, false, 0);
    // Setting up all the buttons specifications
    button0 = new Button(1, width/2, height/2 -232, 80, 30, #000000, #222222, #777777);
    buttonP1 = new Button(0, width/2 + 220, height/2 + 150, 20, 20, #cccccc, #eeeeee, #ffffff);
    buttonP2 = new Button(0, width/2 + 220, height/2 + 200, 20, 20, #cccccc, #eeeeee, #ffffff);
  }
  
  void setup()
  {
    spider.display();
    getLinesGetter();
    initPlayerLines();
  }
  
  void draw()
  {
    // BACKGROUND, TITLE & INFORMATION ----------------------------------------------//
    pushStyle();
    background(#333333);
    
    stroke(#eeeeee);
    line(width/2, 0, width/2, height/2 - 150);
    
    fill(#ffffff);
    textAlign(CENTER, CENTER);
    textSize(68);
    text("SPIDEY!", width/2, height/5-2);
    
    textSize(25);
    text("Choose Player Microphones", width/2, height/2 + 100); 

    textSize(15);
    text("P1", width/2 - 230, height/2 + 148);
    text("P2", width/2 - 230, height/2 + 198);
    
    rectMode(CENTER);
    noStroke();
    rect(width/2, height/2 + 150, 400, 17);
    fill(#222222);
    text("" + player1Line.mixerInfo.getName(), width/2, height/2 + 147);
    
    fill(#ffffff);
    rect(width/2 , height/2 + 200, 400, 17);
    fill(#222222);
    text("" + player2Line.mixerInfo.getName(), width/2, height/2 + 197);
        
        
    // Displaying a warning if both players have the same input line selected   
    if(player1LineIndex == player2LineIndex)
    {
      fill(#ee2222);
      text("You are using the same microphone for P1 and P2!", width/2, height/2 + 240);
    }
    
    // Displaying a confirmation that everything is if the input lines differ
    if(player1LineIndex != player2LineIndex)
    {
      fill(#22ee22);
      text("Good to go!", width/2, height/2 + 230);
    }
    
    
    // BUTTON HANDLING --------------------------------------------------------------- //
    // When start button gets pressed and the players have differing input lines the spidey game gets called and lines sent to it
    if(button0.status == true && player1LineIndex != player2LineIndex)
    {
      try
      { 
        gameMode = new SpideyGame(player1Line.getLine(), player2Line.getLine());
      }
      // Error handling if line(s) is/ are unavailable or if there is an illegal argument
      catch (LineUnavailableException ex) 
      {
        println(ex.toString());
        exit();
      }
      catch (IllegalArgumentException ex)
      {
        println(ex.toString());
        exit();
      }
    }
    
    // Resetting the button status if same microphone selected (game otherwise not playable)
    if(button0.status == true && player1LineIndex == player2LineIndex)
    {
      button0.status = false;
    }
    
    // Button to cycle through the available lines for player one
    if(buttonP1.status == true)
    {
      nextLinePlayer(1);
      buttonP1.status = false;
    }
    
    // Button to cycle through the available lines for player two
    if(buttonP2.status == true)
    {
      nextLinePlayer(2);
      buttonP2.status = false;
    }

    
    // BUTTON DRAWING --------------------------------------------------------------- //
    fill(#000000);
    noStroke();
    button0.draw();  
    buttonP1.draw();
    buttonP2.draw();
    spider.display();

    textSize(20);
    text("START", width/2, height/2 - 235);
    
    popStyle();
  }
   
  // Enumerates all available microphones
  void getLinesGetter()
  {
    // calls the Java AudioSystem for all the mixer infos and stores them in an array
    Mixer.Info[] mixerInfos = AudioSystem.getMixerInfo();
    for (Mixer.Info info: mixerInfos)
    {
      // gets the lines in the mixer
      Mixer m = AudioSystem.getMixer(info);
      Line.Info[] lineInfos = m.getTargetLineInfo();
      
      // Checks if line is a microphone 
      if(lineInfos.length>=1 && lineInfos[0].getLineClass().equals(TargetDataLine.class)) 
      {
        // creates a TargetDataLine for each available line
        for (Line.Info lineInfo:lineInfos)
        {
          TargetDataLine line;
          
          // stores the information about the line in the lineGetter array
          try 
          {
            line = (TargetDataLine) m.getLine(lineInfo);
            lineGetters.add(new LineGetter(m, info, lineInfo));
          } 
          catch (Exception e) 
          {
          println("Skipping device: "+e);
        }
      }
    }
  }
  
  // printing out all the found and available microphones (mostly for debugging)
  println("I have found the following microphones: "); 
  for (LineGetter l : lineGetters) 
  {
    println(l.mixerInfo.getName());
    try 
    {
      println(l.getLine());
     }
     catch (Exception e) 
     {
       System.err.println(e);
     }
    }
  }
  
  // setting up initial player lines
  void initPlayerLines()
  {
    player1LineIndex = 0;
    player2LineIndex = 0;
    player1Line = lineGetters.get(player1LineIndex);
    player2Line = lineGetters.get(player2LineIndex);
  }
  
  // method to point the player lines to the next available
  void nextLinePlayer(int player)
  {    
    // checking if the method is called for the first player
    if(player == 1)
    {
      player1LineIndex ++;
      println("player 1 Index; " + player1LineIndex);
      
      // checking if the line index is bigger then the amount of available lines and resetting it to index 0 if so
      if(player1LineIndex > lineGetters.size() -1)
      {
        println("No more Lines. Returning to first.");
        player1LineIndex = 0;
        player1Line = lineGetters.get(player1LineIndex);
        return;
      }  
      player1Line = lineGetters.get(player1LineIndex);
    }
    
    // checking if method is called for second player
    // same as above but for all the player 2 variables
    if(player == 2)
    {
      player2LineIndex ++;
      println("player 1 Index; " + player2LineIndex);
      if(player2LineIndex > lineGetters.size() -1)
      {
        println("No more Lines. Returning to first.");
        player2LineIndex = 0;
        player2Line = lineGetters.get(player2LineIndex);
        return;
      }  
      player2Line = lineGetters.get(player2LineIndex);
    }
  }
  
  // handling the mouse clicks for the buttons.
  void mouseClicked()
  {
    button0.mouseClicked();
    buttonP1.mouseClicked();
    buttonP2.mouseClicked();
  }
}
