class Spider extends Sprite
{
  GameWorld gameWorld;
  
  // input toggles to controll game with arrow keys (debug)
  boolean upPressed;
  boolean leftPressed;
  boolean downPressed;
  boolean rightPressed;

  boolean wPressed;
  boolean aPressed;
  boolean sPressed;
  boolean dPressed;
  
  //spider graphic
  PImage[] s_anim = new PImage[8];

  //spidey movement timer
  float timeNow, timePre = 0, timeDel; //variables to store different moments in time. now, past, difference
  float springTimer = 5;
  
  //array which represents the spiders silk:
  int n = 2; //number of links
  Spring[] springs = new Spring[n]; //array for active spiders silk
  Spring[] _springs; //temporary storage of new silk
  Spring[] oldSprings; //array to display old/inactive silk in

  Spider(GameWorld _gameWorld)  
  {
    super(loadFrames("Spidey", 8, 100, 50), 0, 0, 0.1, false, 0);
    gameWorld = _gameWorld;
    //initiate spiders-silk – store n links in an array
    for (int i=0; i<n; i++) 
    {
      springs[i] = new Spring(i);
    }
  }
  
  //draws silk
  void display() 
  {   
    checkInputToggles(); //debug
    
    //draw silk graphics and envoke velocity-update-function for each link
    stroke(255);
    for (Spring s : springs) {
      s.updateVel(); //acceleration gets updated in this method also
      line(s.origin.x, s.origin.y, s.p.x, s.p.y); // silk graphics
      if (s.index == n-1) {
        this.xPos = s.p.x - this.sprite_width/2;
        this.yPos = s.p.y;
        super.display();
      } 
    }
     // draw inactive silk after silk collided with the games map for aesthetical reasons
     if(oldSprings != null && oldSprings[0] != null){ //check if the oldSprings array is instantiated and if its entries are not empty - first entry is enough
       for(Spring oS : oldSprings){
         line(oS.origin.x, oS.origin.y, oS.p.x, oS.p.y); // draw lines for old silk
     }
   }
    gameWorld.wind.mult(0); // RESET external forces, i.e. wind
    
  }
  
  //handles spiders automated movement downwards 
  void move(){
    timeNow = millis();
    timeDel = timeNow-timePre;
    // if springTime counts down to 0 new Springs get created and appended to the silk array.
    // otherwise springTime counts down
    if (springTimer <= 0)
    {
      springs = (Spring[]) append(springs, new Spring(n));
      n++;
      springTimer = 190;
    } else
    {
      println("timeDel: "+ timeDel);
      springTimer = springTimer-timeDel;
    }
    timePre = timeNow;
  }

  void silkCollision() {
    int lastLinkCollided = 0;

     //for each spring in the springs array: check for collision
    for(Spring s : springs){
       int xCoord = int(s.p.x);
       int yCoord = int(s.p.y);
    
          int linkPos = gameWorld.map.pixels[ yCoord * gameWorld.map.width + xCoord]; 
 
       if  (((linkPos  >> 16) & 0xff) == 255 && (((linkPos  >> 8) & 0xff) == 0)) { 
         if(s.index != springs.length-1){ //make sure that the collided link is not the spider
           lastLinkCollided = s.index+1; //+1, because the last collided link is the origin of the folowing link
         }
       }
    }

   // adjust number of links in silk if collision happens,
   // also add the then inactive part of the silk in the oldSilk array
   if(lastLinkCollided != 0){
     int offset = 0;
      
     //make new temp. array that can store every link from the collided one to the end
     _springs = new Spring[springs.length - lastLinkCollided];
     
      //prepare array-length for old silk
      if(oldSprings == null){
        //if the arary is empty make array to store old links
        oldSprings = new Spring[lastLinkCollided];
      } else {  
        //if there are already old links stored in the array, expand it by the number of those to be added
        offset = oldSprings.length; //
        oldSprings = (Spring[])expand(oldSprings, oldSprings.length + (springs.length - _springs.length)); 
      }
      
      //store old silk
      for(int i = 0; i < lastLinkCollided; i++){
        oldSprings[offset + i] = springs[i];
        oldSprings[offset + i].index = offset + i; //change index properties for all springs 
      }
       
      //store all links after the collided one in the _springs array
      for(int i = lastLinkCollided; i < springs.length; i++){
        _springs[i-lastLinkCollided] = springs[i]; 
        _springs[i-lastLinkCollided].index = i-lastLinkCollided; //change index properties
      }
      
      //copy temp. _springs array into main springs array
      springs = new Spring[_springs.length];
      arrayCopy(_springs, springs);
      n = springs.length;
    }
  }

  // key presses handling for alternative controlls / debugging purposes
  void keyPressed()
  { 

    if (key == CODED) 
    { 
      if (keyCode == LEFT)
      {
        leftPressed = true;
      }

      if (keyCode == RIGHT) 
      {
        rightPressed = true;
      }
    }
  }


  void keyReleased()
  { 
    if (key == CODED)
    {     
      if (keyCode == LEFT)
      {
        leftPressed = false;
      }

      if (keyCode == RIGHT)
      {
        rightPressed = false;
      }
    }
  }

  void checkInputToggles()
  {
    if (leftPressed == true)
    {   

      gameWorld.wind.add(-3.5, 0.0); 
      ellipse(50, height/2, 50, 50);
    }

    if (rightPressed == true)
    {
      gameWorld.wind.add(3.5, 0.0);
      ellipse(width - 50, height/2, 50, 50);
    }
  }

  //--------------------------------------------------

  class Spring{
  int index;
  
  float restLen; //restLength
  float currentLen; 
  float displacement;
  //float mass;
   
  PVector origin, // start of spring
          p, // end of spring
          v = new PVector(0,0), //velocity
          sF = new PVector(0,0); //spring force
     
  Spring(int _index){
     
    index = _index;
    restLen = 1;
    
    //define start and endpoint of a spring 
    if(index == 0){
      origin = new PVector(width/1.5,height/2.95);      
    } else {
      origin = springs[index-1].p;
    }
    p = origin.copy().add(new PVector(0,restLen));
  }
  
  //update velocity-vector and add it to p
  void updateVel(){
 
    updateSpringForce();
    
    v.add(sF);
    v.add(gameWorld.wind);  
    v.add(gameWorld.gravity);
    //friction
    gameWorld.friction = v.copy();
    float c = 0.65;  //friction constant: change to vary flow-effect
    gameWorld.friction = gameWorld.friction.mult(-1.0).mult(c); 
    v.add(gameWorld.friction);
    p.add(v);
  }
  
  void updateSpringForce(){
    //bouncing  
    currentLen = PVector.sub(p, origin).mag();
    
    displacement =  currentLen -restLen;
    
    //force direction
    sF = PVector.sub(origin, p).normalize(); 
    
    //scale force direction with displacement * spring constant k
    float k = 0.55; 
    sF.mult(k*displacement);
  }
  
 }//SPRING CLASS ENDS HERE  
}//SPIDEY CLASS ENDS HERE
