// the game objects are sprites, meaning they are animated by showing the single frames of an image array in quick succesion, creating an animation

class Sprite {
  float speed;
  float frameIndex;
  int displayFrame;
  PImage[] frames;
  float xPos, yPos;
  boolean left;
  int type;
  float sprite_width, sprite_height;


  Sprite(PImage[] frames, float xPos, float yPos, float speed, boolean left, int type) {
    this.speed = speed;
    this.frames = frames;
    this.xPos = xPos;
    this.yPos = yPos;
    this. frameIndex = 0;
    this.displayFrame = 0;
    this.left = left;                         // decides wether the sprite faces to the left or the right
    this.type = type;                         // the type defines whether the sprite is an enemy, a plus point for Player 1 or Player 2
    this.sprite_width = frames[1].width;
    this.sprite_height = frames[1].height;
  }

  void display() {
    if (left == false) {
      image(frames[displayFrame % frames.length], xPos, yPos, sprite_width, sprite_height); //goes through the frames of the array in order, creating an animation
    }
    if (left) {                               // mirrors the sprite vertically so it faces to the other side
      pushMatrix();
      translate(xPos+sprite_width, yPos);
      scale(-1, 1);
      image(frames[displayFrame % frames.length], 0, 0, sprite_width, sprite_height);
      popMatrix();
    }
  }

  void setPos(float x, float y) {
    this.xPos = x;
    this.yPos = y;
  }

  void animate() {                              // the pace of the animation is controlled according to speed
    frameIndex = frameIndex + speed;
    displayFrame = floor(frameIndex);
    if (left) {                              
      xPos = xPos - speed;
      if (xPos < - sprite_width) {               // makes sure the sprite is reset once it exits the frame so it comes back around
        xPos = width;
      }
    } else {
      xPos = xPos + speed;
      if (xPos > width) {
        xPos = - sprite_width;
      }
    }
  }

  //collsion between two Sprites, represented by two Rectangles / Rectangle/ Rectangle collision


  boolean spriteCollision (Sprite r1, Sprite r2) {

    if (r1.xPos + r1.sprite_width >= r2.xPos   &&    // r1 right edge past r2 left
      r1.xPos <= r2.xPos + r2.sprite_width     &&    // r1 left edge past r2 right
      r1.yPos + r1.sprite_height >= r2.yPos     &&    // r1 top edge past r2 bottom
      r1.yPos <= r2.yPos + r2.sprite_height) {        // r1 bottom edge past r2 top

      return true;
    }
    return false;
  }
}
